@@ -7,7 +7,8 @@ test_that("add_mining_field_attributes writes MiningField attributes properly",
   attributes <- data.frame(
     c("active", 1.1, "asIs"),
     c("active", 2.2, "asIs"),
-    c("active", NA, "asMissing")
+    c("active", NA, "asMissing"),
+    stringsAsFactors = TRUE
   )
   rownames(attributes) <- c(
     "usageType", "missingValueReplacement",
