@@ -19,36 +19,36 @@ library(forecast)
 data(iris)
 data(audit)
 data("WWWusage")
-iris_p <- read.csv("iris.csv")
+iris_p <- read.csv("iris.csv", stringsAsFactors = TRUE)
 audit <- na.omit(audit)
 audit_factor <- audit
 audit_factor[, 13] <- as.factor(audit_factor[, 13])
-elnino <- read.csv("elnino.csv")
-heart <- read.csv("heart.csv")
-glm_issue3543_data <- read.csv("glm_issue3543_data.csv")
-credit_class <- read.csv("credit_class.csv")
-covtype2 <- read.csv("covtype2.csv", header = TRUE)
-credit <- read.csv("credit.csv")
-credit_class_01 <- read.csv("credit_class_01.csv")
-audit_nor_logical <- na.omit(read.csv("audit_nor_logical.csv"))
-audit_nor <- na.omit(read.csv("audit_nor.csv"))
-audit_nor_fake_logical <- na.omit(read.csv("audit_nor_fake_logical.csv"))
-random_data_small <- read.csv("random_data_small.csv")
-iris_nor <- read.csv("iris_nor.csv")
-bank <- na.omit(read.csv("bank.csv"))
-audit_r_build_in <- na.omit(read.csv("audit_r_build_in.csv"))
-insurance <- na.omit(read.csv("insurance.csv"))
-iris_bin <- read.csv("iris_bin.csv")
-house_votes <- na.omit(read.csv("house_votes_84.csv"))
-iris_mini_dot <- read.csv("iris_mini_dot.csv")
-petfood <- read.csv("petfood.csv")
-job_cat <- read.csv("job_cat.csv")
-job_cat_index <- read.csv("job_cat_index.csv")
-iris_nor_logical <- read.csv("iris_nor_logical.csv")
-factor_40k <- read.csv("factor_40k.csv")
-numeric_10k <- na.omit(read.csv("numeric_10k.csv"))
-factor_10k <- read.csv("factor_10k.csv")
-numeric_no_na_10k <- read.csv("numeric_no_na_10k.csv")
+elnino <- read.csv("elnino.csv", stringsAsFactors = TRUE)
+heart <- read.csv("heart.csv", stringsAsFactors = TRUE)
+glm_issue3543_data <- read.csv("glm_issue3543_data.csv", stringsAsFactors = TRUE)
+credit_class <- read.csv("credit_class.csv", stringsAsFactors = TRUE)
+covtype2 <- read.csv("covtype2.csv", header = TRUE, stringsAsFactors = TRUE)
+credit <- read.csv("credit.csv", stringsAsFactors = TRUE)
+credit_class_01 <- read.csv("credit_class_01.csv", stringsAsFactors = TRUE)
+audit_nor_logical <- na.omit(read.csv("audit_nor_logical.csv", stringsAsFactors = TRUE))
+audit_nor <- na.omit(read.csv("audit_nor.csv", stringsAsFactors = TRUE))
+audit_nor_fake_logical <- na.omit(read.csv("audit_nor_fake_logical.csv", stringsAsFactors = TRUE))
+random_data_small <- read.csv("random_data_small.csv", stringsAsFactors = TRUE)
+iris_nor <- read.csv("iris_nor.csv", stringsAsFactors = TRUE)
+bank <- na.omit(read.csv("bank.csv", stringsAsFactors = TRUE))
+audit_r_build_in <- na.omit(read.csv("audit_r_build_in.csv", stringsAsFactors = TRUE))
+insurance <- na.omit(read.csv("insurance.csv", stringsAsFactors = TRUE))
+iris_bin <- read.csv("iris_bin.csv", stringsAsFactors = TRUE)
+house_votes <- na.omit(read.csv("house_votes_84.csv", stringsAsFactors = TRUE))
+iris_mini_dot <- read.csv("iris_mini_dot.csv", stringsAsFactors = TRUE)
+petfood <- read.csv("petfood.csv", stringsAsFactors = TRUE)
+job_cat <- read.csv("job_cat.csv", stringsAsFactors = TRUE)
+job_cat_index <- read.csv("job_cat_index.csv", stringsAsFactors = TRUE)
+iris_nor_logical <- read.csv("iris_nor_logical.csv", stringsAsFactors = TRUE)
+factor_40k <- read.csv("factor_40k.csv", stringsAsFactors = TRUE)
+numeric_10k <- na.omit(read.csv("numeric_10k.csv", stringsAsFactors = TRUE))
+factor_10k <- read.csv("factor_10k.csv", stringsAsFactors = TRUE)
+numeric_no_na_10k <- read.csv("numeric_no_na_10k.csv", stringsAsFactors = TRUE)
 
 
 xgb_tmp_01_save <- tempfile()
@@ -138,7 +138,9 @@ zmz_transform_audit <- function(box_obj) {
 
 
 # schema <- XML::xmlSchemaParse("pmml-4-4_xslt_20190830_10.5.0.0.xsd")
-schema <- XML::xmlSchemaParse("pmml-4-4.xsd")
+# schema <- XML::xmlSchemaParse("pmml-4-4.xsd")
+schema <- XML::xmlSchemaParse("pmml-4-4_statespace.xsd")
+
 
 test_that("TimeSeries/Arima PMML validates against schema", {
   fit <- Arima(WWWusage, order = c(1, 0, 1))
@@ -303,7 +305,7 @@ test_that("GeneralRegressionModel/glmnet PMML validates against schema", {
   expect_equal(validate_pmml(p_fit, schema), 0)
 
 
-  x <- data.frame(replicate(20, rnorm(1000)))
+  x <- data.frame(replicate(20, rnorm(1000)), stringsAsFactors = TRUE)
   y <- rnorm(1000)
   box_obj <- xform_wrap(x)
   box_obj <- xform_min_max(box_obj, xform_info = "column1->d_X1")
@@ -480,7 +482,7 @@ test_that("MiningModel/randomForest PMML validates against schema", {
   fit <- randomForest(SEPAL_LE ~ ., data = iris_nor, ntree = 9)
   expect_equal(validate_pmml(pmml(fit), schema), 0)
 
-  iris_nor_logical <- read.csv("iris_nor_logical.csv")
+  iris_nor_logical <- read.csv("iris_nor_logical.csv", stringsAsFactors = TRUE)
   iris_nor_logical[, 5] <- as.factor(iris_nor_logical[, 5])
   fit <- randomForest(SEPAL_LE ~ ., iris_nor_logical, ntree = 7)
   expect_equal(validate_pmml(pmml(fit), schema), 0)
